// C++ code
// basicdefs.cpp
// Author: Yun S. Song


#include <iostream>
#include <vector>
#include <math.h>
#include "basicdefs.h"
#include <stdio.h>
#include <ctype.h>
#include <set>
#include <fstream>
#include <algorithm>

extern long randomseed;

// description of the program
void progdesc(){
  printf("SYNOPSIS:  This program approximates the sampling probability of an ordered\n");
  printf("           configuration using the asymptotic expansion discussed in\n");
  printf("           Jenkins and Song (2009).\n");
  printf("           Either an infinite-alleles or a finite-alleles PIM may be used.\n\n");
  printf("           asf [-k K] [-l L] [-a a_1 ... a_K] [-b b_1 ... b_L]\n");
  printf("               [-P P_1 ... P_K] [-Q Q_1 ... Q_L] [-tA thetaA] [-tB thetaB]\n");
  printf("               [-r rho1 rho2 ... ] [-h]\n");
  printf("               -c c_{11}...c_{1L} c_{21}...c_{2L} ... c_{K1}...c_{KL}\n");
  printf("                 \n\n");
  printf("OPTIONS:  \n");
  printf("   -k #             Number of alleles at locus A.  (default = 2)\n");
  printf("                    This option is ignored if -a option is\n");
  printf("                    specified.   Either -a or -k must be specified.\n\n");
  printf("   -l #             Number of alleles at locus B.  (default = 2)\n");
  printf("                    This option is ignored if -b option is\n");
  printf("                    specified.  Either -b or -l must be specified.\n\n");
  printf("   -a  a_1 ... a_K  Configuration for left-half fragments.\n\n");
  printf("   -b  b_1 ... b_L  Configuration for right-half fragments.\n\n");
  printf("   -c  c_11...c_KL  Configuration for full gametes.\n\n");
  printf("   -P  P_1 ... P_K  Mutation transition matrix for locus A.\n");
  printf("                    If either -P or -Q is specified, then the PIM model is\n");
  printf("                    assumed.\n\n");
  printf("   -Q  Q_1 ... Q_K  Mutation transition matrix for locus B.\n\n");
  printf("                    If either -P or -Q is specified, then the PIM model is\n");
  printf("                    assumed.\n\n");
  printf("   -r  # ... #      Recombination rates (default = 50).\n\n");
  printf("   -tA #            Mutation rate at locus A (default = 0.001).\n\n");
  printf("   -tB #            Mutation rate at locus B (default = 0.001).\n\n");
  printf("   -d  #            Display option.\n");
  printf("                    Input a number greater than 0 for condensed output.\n\n");
  printf("   -e               Exact computation of q_2.  If this option is not specified,\n");
  printf("                    the recursive part of q_2 will not be computed.\n\n");
  printf("   -h               Print this page.\n\n");
  exit(1);
}

// get options from command line

void getoptions(int argc, char *argv[]){
  extern vector<int> aVec, bVec, cVec;
  extern int noAllelesA;
  extern int noAllelesB;
  extern double thetaA, thetaB;
  extern vector<double> P, Q, rho;
  extern int display;
  extern bool exactq2; 
  extern int aTotal, bTotal, cTotal;
  if(argc < 2){
    progdesc();
  }
  else{
    for(int iI=1; iI < argc; iI++){
      if(argv[iI][0] != '-'){
	cerr << "ERROR: Options must begin with '-'.\n\n";
	//progdesc();
	exit(1);
      }
      else{
      	switch(argv[iI][1]){
	case 'k':{
	  noAllelesA=atoi(argv[iI+1]);
	  iI++;
	  break;
	}
	case 'l':{
	  noAllelesB=atoi(argv[iI+1]);
	  iI++;
	  break;
	}
	case 'a':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
	    aVec.push_back(atoi(argv[iI+1]));
	    aTotal += atoi(argv[iI+1]);
	    iI++;
	  }
	  break;
	}
	case 'b':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
	    bVec.push_back(atoi(argv[iI+1]));
	    bTotal += atoi(argv[iI+1]);
	    iI++;
	  }
	  break;
	}
	case 'c':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
	    cVec.push_back(atoi(argv[iI+1]));
	    cTotal += atoi(argv[iI+1]);
	    iI++;
	  }
	  break;
	}
	case 't':{
	  if(argv[iI][2]=='A'){
	    thetaA=atof(argv[iI+1]);
	    iI++;
	  }
	  else{ 
	    if(argv[iI][2]=='B'){
	      thetaB=atof(argv[iI+1]);
	      iI++;
	    }
	    else{
	      cerr << "Unknown -t? option.\n";
	      exit(1);
	    }
	  }
	  break;
	}
	case 'P':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
		P.push_back(atof(argv[iI+1]));
		iI++;
	  }
	  break;
	}
	case 'Q':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
		Q.push_back(atof(argv[iI+1]));
		iI++;
	  }
	  break;
	}
	case 'r':{
	  while(iI+1 != argc && argv[iI+1][0] != '-'){
	    rho.push_back(atof(argv[iI+1]));
	    iI++;
	  }
	  break;
	}
	case 'd':{
	  display = atoi(argv[iI+1]);
	  iI++;
	  break;
	}
	case 'e':{
	  exactq2 = true;
	  break;
	}
	case 'h':{
	  progdesc();
	  break;
	}
	default:
	  printf("ERROR: Unknown argument type.\n\n");
	  //progdesc();
	  exit(1);
	  break;
	}
      }
    }
  }
}





// given a matrix c, get a marginal vector for locus loc
vector<int> marginalVec(vector<vector<int> > &matrix, char loc){
  vector<int> vec;
  int iI, iJ;
  if(loc=='A'){ // first locus
    vec.resize(matrix.size());
    for(iI=0; iI < matrix.size(); iI++){
      vec[iI]=0;
      for(iJ=0; iJ < matrix[iI].size(); iJ++){
	vec[iI] += matrix[iI][iJ];
      }
    }
  }
  else{ // second locus
    vec.resize(matrix[0].size());
    for(iJ=0; iJ < matrix[0].size(); iJ++){
      vec[iJ]=0;
      for(iI=0; iI < matrix.size(); iI++){
	vec[iJ] += matrix[iI][iJ];
      }
    }
  }
  return vec;
}

vector<int> addVec(vector<int> & vec1, vector<int> & vec2){
  if(vec1.size() != vec2.size() ){
    cerr << "Two vectors of different size cannot be added." << endl;
    exit(1);
  }
  vector<int> newVec(vec1.size());
  for(int i=0; i< vec1.size(); i++){
    newVec[i]=vec1[i]+vec2[i];
  }
  return newVec;
}


vector<int> concatenate(vector<int>& v1, vector<int> & v2){
  vector<int> tmp(v1.size()+v2.size());
  int i;
  for(i=0; i < v1.size(); i++){
    tmp[i]=v1[i];
  }
  for(i=0; i < v2.size(); i++){
    tmp[i+v1.size()]=v2[i];
  }
  return tmp;
}
